/**********************************************************************
  PROJECT: Controls Demo
  Source: con_demo.cpp
  Chapter reference: 6
***********************************************************************
  Description:
    Demonstration of basic controls and control processing
 
  Topics:
    1. Creating a child window with controls
	2. Destroying the child window
	3. Radio button control and operation
	4. Checkbox control and operation
	5. Scrollbar control and operation
***********************************************************************/

#include <windows.h>

// Prototypes
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK ChildWndProc (HWND, UINT, WPARAM, LPARAM) ;

// Constants for child windows and controls
#define  CHILD1      1001
#define  CREATEWIN   102
#define  DESTROYWIN  103
#define  RADGROUP    104
#define  RADIO1      201
#define  RADIO2      202
#define  RADIO3      203
#define  CHKBOX1     301
#define  CHKBOX2     302
#define  SCROLLBAR   401
#define  SCRBARWIN   501
#define  FRAME       502

//**********************************
//          global data
// *********************************
HWND        hwnd, hChild ;
HINSTANCE   pInstance;
HDC         hdc, childDc;
bool		childStatus = FALSE;

//***********************************
//            WinMain()
//***********************************
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;

     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = sizeof (WORD);
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = NULL;
     wndclass.lpszClassName = szAppName ;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

     RegisterClassEx (&wndclass) ;

     pInstance = hInstance;


     hwnd = CreateWindow(
		         szAppName,
                "Basic Controls Demo",
                WS_OVERLAPPEDWINDOW,
                CW_USEDEFAULT, CW_USEDEFAULT,
                CW_USEDEFAULT, CW_USEDEFAULT,
                NULL, NULL, hInstance, NULL) ;

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }

//*********************************
//        Windows Procedure
// ********************************
LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam,
						 LPARAM lParam) {

	 static HWND  hwndButton1;     // Handle to first button
   
     WNDCLASSEX  chiclass ;
     static int butCode, butNote;   // button id and notification

	 switch (iMsg) {

         case WM_CREATE:
           hdc = GetDC (hwnd) ;
           // The system monospaced font is selected
           SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT)) ;
 		   // Display a labeled pushbutton on the main window
				hwndButton1 = CreateWindow (
					"BUTTON",               // Control class name
					"Create Child",         // Button name text
					WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
					20, 20,                 // x and y location
					140,                    // Window width
					30,                     // Window height
					hwnd,                   // Handle to parent window
					(HMENU) CREATEWIN,      // Child window id.
					pInstance,              // Instance handle 
					NULL) ;
		   return 0 ;

		 case WM_COMMAND:
			 butCode = LOWORD (wParam);
			 butNote = HIWORD (wParam);

			 switch (butCode) {

				case CREATEWIN:
					if(butNote == BN_CLICKED && !childStatus) {
					// Define and register the child class
						chiclass.cbSize        = sizeof (chiclass) ;
						chiclass.style         = CS_HREDRAW | CS_VREDRAW \
							                     | CS_OWNDC;
						chiclass.lpfnWndProc   = ChildWndProc ;
						chiclass.cbClsExtra    = 0 ;
						chiclass.cbWndExtra    = 0 ;
						chiclass.hInstance     = pInstance ;
						chiclass.hIcon         = NULL;
						chiclass.hCursor       = LoadCursor (NULL, IDC_CROSS) ;
						chiclass.hbrBackground = (HBRUSH) GetStockObject (LTGRAY_BRUSH) ;
						chiclass.lpszMenuName  = NULL;
						chiclass.lpszClassName = "ChildWindow" ;
						chiclass.hIconSm       = NULL;

						RegisterClassEx (&chiclass) ;
						// Create and display child window
						hChild = CreateWindow (
						"ChildWindow",
						NULL,				// no caption
						WS_CHILD | WS_THICKFRAME | WS_VISIBLE ,
						120, 80,         // x and y of window location
						400, 200,        // x and y of window size
						hwnd,            // handle to the parent window 
						(HMENU) CHILD1,  // child window designation
						pInstance,       // program instance
						NULL) ;
						// Update child
					    UpdateWindow(hChild);
						childStatus = TRUE;     // Set child window displayed switch
					} 
					return 0;
				} 
			return 0;

         case WM_DESTROY:
          PostQuitMessage (0) ;
           return 0 ;

	 }

     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
} 


//**********************************
//    child windows procedure
//**********************************
LRESULT CALLBACK ChildWndProc (HWND hChild, UINT iMsg, WPARAM wParam,
							   LPARAM lParam) {

	static HWND  hwndChildBut1;     // Handle to child's button
	static HWND  hwndRadio1, hwndRadio2, hwndRadio3;
	static HWND  hwndChkBx1, hwndChkBx2;
	static HWND  hwndSB, hwndVal;
	static HWND  hwndGrpBox1;
	static HWND  hwndFrame;

    int          buttonID, buttonNotCode;  //
	LRESULT      butMsg;

	SCROLLINFO   scinfo;
	
    static int   sbPos = 50;      // Scrollbar position
    static char  szBuffer[5];     // Buffer for text string  

	switch (iMsg) {

	   case WM_CREATE:
          // make sure that child window is not displayed
		  if(childStatus)
		    return 0;

          childStatus = TRUE;
		  childDc = GetDC(hChild);
          SelectObject (childDc, GetStockObject (SYSTEM_FIXED_FONT)) ;

		  // Place destroy button on child window
		  hwndChildBut1 = CreateWindow (
				"BUTTON",               // Control class name
				"Destroy Child",         // Button name text
				WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				20, 20,                 // x and y location
				150,                    // Window width
				30,                     // Window height
				hChild,                 // Handle to parent window
				(HMENU) DESTROYWIN,     // Child window id.
				pInstance,              // Instance handle (global variable)
				NULL) ; 

		   //**************************
		   //   create radio buttons
		   //      in group box
		   //**************************
		   // 3 radio buttons are enclosed in a group box
           hwndGrpBox1 = CreateWindow (
			   "BUTTON",			   // Control class name
			  "Radio Group",           // Button name text
               WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
               240,                    // x coordinate of window location
               20,                     // y coordinate
               140, 160,               // button size
               hChild,                 // Handle to parent window
			  (HMENU) RADGROUP,        // Child window id.
               pInstance,              // Instance handle (global variable)
			   NULL) ; 
		   
		   // Radio button # 1
           hwndRadio1 = CreateWindow (
			   "BUTTON",			   // Control class name
			  "Radio 1",               // Button name text
               WS_CHILD | WS_VISIBLE | BS_RADIOBUTTON,
               260,                    // x coordinate of window location
               50,                     // y coordinate
               100, 30,                // button size
               hChild,                 // Handle to parent window
			  (HMENU) RADIO1,          // Child window id.
               pInstance,              // Instance handle (global variable)
			   NULL) ;                 // Pointer to additional data (none)
			 // Set first radio button ON
			 SendMessage(hwndRadio1, BM_SETCHECK, 1, 0L);

		   // Radio button # 2
           hwndRadio2 = CreateWindow ("BUTTON",  // Control class name
			  "Radio 2",                            // Button name text
               WS_CHILD | WS_VISIBLE | BS_RADIOBUTTON,
               260,                    // x coordinate of window location
               90,                     // y coordinate
               100, 30,                // button size
               hChild,                 // Handle to parent window
			  (HMENU) RADIO2,          // Child window id.
               pInstance,              // Instance handle (global variable)
			   NULL) ;                 // Pointer to additional data (none)

		   // Radio button # 3
           hwndRadio3 = CreateWindow ("BUTTON",  // Control class name
			  "Radio 3",                            // Button name text
               WS_CHILD | WS_VISIBLE | BS_RADIOBUTTON,
               260,                    // x coordinate of window location
               130,                    // y coordinate
               100, 30,                // button size
               hChild,                 // Handle to parent window
			  (HMENU) RADIO3,          // Child window id.
               pInstance,              // Instance handle (global variable)
			   NULL) ;                 // Pointer to additional data (none)

		   //**************************
		   //   create checkboxes
		   //**************************
		   // Chechbox # 1
           hwndChkBx1 = CreateWindow ("BUTTON",  // Control class name
			  "ChkBx 1",                            // Button name text
               WS_CHILD | WS_VISIBLE | BS_3STATE,
			   20,                      // x coordinate of window location
 			   80,                     // y coordinate
               90, 30,                 // button dimensions
               hChild,                  // Handle to parent window
			   (HMENU) CHKBOX1,        // Child window id.
               pInstance,               // Instance handle (global variable)
			   NULL) ;                  // Pointer to additional data (none)
		   // Chechbox # 2
           hwndChkBx2 = CreateWindow ("BUTTON",  // Control class name
			  "ChkBx 2",                            // Button name text
               WS_CHILD | WS_VISIBLE | BS_3STATE,
               120,                      // x coordinate of window location
 			   80,                     // y coordinate
               90, 30,                 // button dimensions
               hChild,                  // Handle to parent window
			   (HMENU) CHKBOX2,         // Child window id.
               pInstance,               // Instance handle (global variable)
			   NULL) ;                  // Pointer to additional data (none)

		   // Draw a frame arround checkboxes using a static class
           hwndFrame = CreateWindow ("STATIC",  // Control class name
				  "",                      // Button name 
                  WS_CHILD | WS_VISIBLE | SS_BLACKFRAME ,
				  10,                      // x coordinate of window location
				  70,                      /// y coordinate
                  210, 50,                 // dimensions
                  hChild,                  // handle to parent window
				  (HMENU) FRAME,           // child window id.
                  pInstance,               // Instance handle (global variable)
				  NULL) ;                  // Pointer to additional data (none) 


		   //**************************
		   //   create scrollbar
		   //**************************
			   // Scrollbar class child window
               hwndSB = CreateWindow ("SCROLLBAR",  // Control class name
				  "",                       // Button name text
                  WS_CHILD | WS_VISIBLE | SBS_HORZ , 
				  20,                       // x coordinate of window location
				  140,                      // y coordinate
                  150, 25,                  // dimensions
                  hChild,                   // Handle to parent window
				  (HMENU) SCROLLBAR,        // Child window id.
                  pInstance,                // Instance handle (global variable)
				  NULL) ;                   // Pointer to additional data (none)


			   // Store parameters in SCROLLINFO structure members
               scinfo.cbSize = sizeof(SCROLLINFO);
			   scinfo.fMask  = SIF_POS | SIF_RANGE | SIF_PAGE;
			   scinfo.nMin   = 0;
			   scinfo.nMax   = 99;
			   scinfo.nPage  = 0;
			   scinfo.nPos   = 50;

               // set information 
			   SetScrollInfo(hwndSB, SB_CTL, &scinfo, TRUE); 

			   /* Alternatively, the following functions could have been
			      used:
			   // Set scrollbar range and initial slide position
			   SetScrollRange(hwndSB, SB_CTL, 0, 99, TRUE);
			   //               |        |    |  |     |
               //               |        |    |  |     |__ redraw flag (do it)
			   //    			|        |    |  |________ maximum position
			   //				|        |    |____________ minimum poisiton
			   //   			|        |__________ scrollbar is a control
			   //				|___________________ handle to the scrollbar
			   
			    SetScrollPos(hwndSB, SB_CTL, 50, TRUE);
			   //               |        |    |   |
               //               |        |    |   |____ redraw flag (do it)
			   //               |        |    |________ slide poisiton
			   //               |        |__________ scrollbar is a control
			   //				|___________________ handle 
               */

               // Create a window for displaying scrollbar slide position
               hwndVal = CreateWindow (
				   "STATIC",			   // Control class name
				  "50",                    // Button name 
                  WS_CHILD | WS_VISIBLE | SS_CENTER |\
                  WS_DLGFRAME,
				  190,                     // x coordinate of window location
				  140,                     // y coordinate
                  30, 25,                  // dimensions
                  hChild,                  // handle to parent window
				  (HMENU) SCRBARWIN,       // child window id.
                  pInstance,               // Instance handle (global variable)
				  NULL) ;                  // Pointer to additional data (none) 

		  return 0;

	   case WM_COMMAND:

			 buttonID = LOWORD (wParam);
			 buttonNotCode = HIWORD (wParam);

			 switch (buttonID) {

				 if(buttonNotCode <= 1)
					 return 0;
            
				case DESTROYWIN:        
					if(buttonNotCode == BN_CLICKED) {
						childStatus = FALSE;
						DestroyWindow(hChild);
						UpdateWindow(hwnd);
					} 
		 	        break;

				  // Radio button # 1 action
				case RADIO1:
                   // Set radio button ON
                   SendMessage(hwndRadio1, BM_SETCHECK, 1, 0L);
                   SendMessage(hwndRadio2, BM_SETCHECK, 0, 0L);
				   SendMessage(hwndRadio3, BM_SETCHECK, 0, 0L);
				   MessageBeep(0);
				   break;

				  // Radio button # 2 action
			    case RADIO2:
                   // Set radio button ON
                   SendMessage(hwndRadio1, BM_SETCHECK, 0, 0L);
                   SendMessage(hwndRadio2, BM_SETCHECK, 1, 0L);
				   SendMessage(hwndRadio3, BM_SETCHECK, 0, 0L);
	               MessageBeep(0);
   				   break;

				  // Radio button # 3 action
				case RADIO3:
                   // Set radio button ON
                   SendMessage(hwndRadio1, BM_SETCHECK, 0, 0L);
                   SendMessage(hwndRadio2, BM_SETCHECK, 0, 0L);
				   SendMessage(hwndRadio3, BM_SETCHECK, 1, 0L);
                   MessageBeep(0);
 				   break;

				 // Checkbox # 1 action:
				 // if checked, change to gray
				 // if gray, change to unchecked
                 // if unchecked, change to checked/ 
				 case CHKBOX1:
                   butMsg = (SendMessage(hwndChkBx1, BM_GETCHECK, 0, 0L));
				   if (butMsg == BST_CHECKED)
                       SendMessage(hwndChkBx1, BM_SETCHECK, \
					               BST_INDETERMINATE, 0L);
				   else if (butMsg == BST_INDETERMINATE)
                       SendMessage(hwndChkBx1, BM_SETCHECK,\
					               BST_UNCHECKED, 0L);
				   else
                       SendMessage(hwndChkBx1, BM_SETCHECK, \
					               BST_CHECKED, 0L);
				   break;

				 case CHKBOX2:
                   butMsg = (SendMessage(hwndChkBx2, BM_GETCHECK, 0, 0L));
				   if (butMsg == BST_CHECKED)
                       SendMessage(hwndChkBx2, BM_SETCHECK, \
					               BST_INDETERMINATE, 0L);
				   else if (butMsg == BST_INDETERMINATE)
                       SendMessage(hwndChkBx2, BM_SETCHECK,\
					               BST_UNCHECKED, 0L);
				   else
                       SendMessage(hwndChkBx2, BM_SETCHECK, \
					               BST_CHECKED, 0L);
				   break;

			 } 
		     return 0;

           /**********************************
		           scroll bar intercept
            **********************************/
          case WM_HSCROLL:

             // Make sure action refers to local scrollbar
             // not the Windows scrollbars
			 if(hwndSB == (HWND) lParam) {    
			     switch (LOWORD (wParam))
                 {
                 case SB_LINELEFT:
					 if(sbPos > 0)
					 sbPos--;
					 break;

                 case SB_LINERIGHT:
					 if(sbPos < 99)
					 sbPos++;
					 break;
                 // Processing for user dragging the scroll box
				 case SB_THUMBTRACK:
				 case SB_THUMBPOSITION:
					  sbPos = HIWORD (wParam);
					  break;
				 }
			     // Reset slide
                 SetScrollPos(hwndSB,
				              SB_CTL,
					          sbPos,
						      TRUE);
				 // Update text
				 SetWindowText (hwndVal, itoa (sbPos, szBuffer, 10));
              /*                    |      |     |       |       |
			                        |      |     |       |       |__ radix
									|      |     |       |__ buffer for string
									|      |     |__________ number to convert
									|      |_______ stdlib.h function
									|                (integer to ascii string)
									|_______________ handle to window
              */
			  }
			  return 0;
			 
	   case WM_DESTROY:
          return 0;
	} 
	    return DefWindowProc (hChild, iMsg, wParam, lParam) ;
}

