/**********************************************************************
  PROJECT: Toolbar Demo No 1
  Source: tb1_demo.cpp
  Chapter reference: 5
***********************************************************************
  Description:
    Toolbar demonstrations

  Topics:
    1. Toolbars with standard buttons
	2. Intercepting toolbar button action
	3. Toolbar buttons that reproduce menu commands
***********************************************************************/

#include <windows.h>
#include "resource.h"
#include <commctrl.h>       // common controls header

// The following IDs correspond to the buttons that
// are not mapped to menu entries and to the toolbar itself
#define ID_PROPS   1001
#define ID_HELP    1002
#define ID_TOOLBAR  400

// Prototype for WndProc()
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
HWND CreateAToolBar(HWND); 

// Glogal variables
HINSTANCE   pInstance;                // Handle to instance


/**************************
         WinMain()
***************************/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

	 wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW  ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
     wndclass.lpszClassName = szAppName ;
	 wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

	 pInstance = hInstance;

	 // Call common controls library
	 InitCommonControls();

     RegisterClassEx (&wndclass) ;

     hwnd = CreateWindow (szAppName, 
		                  "Toolbar Demo No 1",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }

/******************************
      Windows Procedure
******************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
     {

     HDC          hdc ;
     HMENU        menu1;       // handle to a menu
     PAINTSTRUCT  ps ;
     TEXTMETRIC   tm ;
     HWND         tbHandle;    // Handle to toolbar

	 /************************************************
	   Array of attributes for buttons in the toolbar
      ************************************************/
	 TBBUTTON tbb[] = {
     STD_FILENEW,  ID_FILE_NEW,      TBSTATE_ENABLED, TBSTYLE_BUTTON,
		                             0, 0, 0, 0,
     STD_FILEOPEN, ID_FILE_OPEN,     TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_FILESAVE, ID_FILE_SAVE,     TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,

     0,            0,                TBSTATE_ENABLED, TBSTYLE_SEP,
	                                 0, 0, 0, 0,
     // Edit group
     STD_COPY,     ID_EDIT_COPY,     TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_CUT,      ID_EDIT_CUT,      TBSTATE_ENABLED, TBSTYLE_BUTTON,
	 	                             0, 0, 0, 0,
     STD_PASTE,    ID_EDIT_PASTE,    TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_FIND,     ID_EDIT_FIND,     TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
	 STD_REPLACE,  ID_EDIT_REPLACE,  TBSTATE_ENABLED, TBSTYLE_BUTTON,
                                     0, 0, 0, 0,
     STD_UNDO,     ID_EDIT_UNDO,     TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_REDOW,    ID_EDIT_REDO,     TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_DELETE,   ID_EDIT_DELETE,   TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,

     0,            0,                TBSTATE_ENABLED, TBSTYLE_SEP,
	                                 0, 0, 0, 0,
     // Print group
     STD_PRINTPRE, ID_PRINT_PREVIEW, TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_PRINT,    ID_PRINT_PRINT,   TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,

     0,            0,                TBSTATE_ENABLED, TBSTYLE_SEP,
	                                 0, 0, 0, 0,

     STD_PROPERTIES,ID_PROPS,        TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     STD_HELP,      ID_HELP,         TBSTATE_ENABLED, TBSTYLE_BUTTON,
	                                 0, 0, 0, 0,
     };

	 static int iMenu;
     static int  cxChar, cyChar, cxClient, cyClient;

     switch (iMsg)
          {
		  case WM_CREATE:
			  /******************************
			       Create a toolbar
               ******************************/
			  tbHandle = CreateToolbarEx (hwnd,
                         WS_CHILD | WS_VISIBLE | CCS_TOP |
						 TBSTYLE_WRAPABLE,
						 ID_TOOLBAR,         // Toolbar ID number
						 0,                  // Number of bitmaps (none)
						 (HINSTANCE)HINST_COMMCTRL,     // Special resource instance for
						                     // standard bitmaps
                         IDB_STD_SMALL_COLOR,// Bitmap resource ID
			             tbb,                // TBBUTTON variable
						 18,                 // Count of buttons + separators
						 0, 0, 0, 0,         // Not required for standard
						                     // buttons
                         sizeof (TBBUTTON)); 


               hdc = GetDC (hwnd) ;
               GetTextMetrics (hdc, &tm) ;
               cxChar = tm.tmAveCharWidth ;
               cyChar = tm.tmHeight ;
               ReleaseDC (hwnd, hdc) ;
               return 0 ;


           case WM_COMMAND:

              menu1 = GetMenu(hwnd);
			    switch (LOWORD (wParam))
                {
				case ID_FILE_NEW:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FILENEW",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;
				   
                 case ID_FILE_OPEN:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FILEOPEN",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case ID_FILE_SAVE:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FILESAVE",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case ID_FILE_EXIT:
					SendMessage(hwnd, WM_CLOSE, 0, 0L);
					return 0;


				   
                 case ID_EDIT_CUT:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_CUT",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case ID_EDIT_COPY:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_COPY",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case ID_EDIT_PASTE:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_PASTE",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_EDIT_FIND:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FIND",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_EDIT_REPLACE:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_REPLACE",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;
                case ID_EDIT_UNDO:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_UNDO",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_EDIT_REDO:
                   MessageBeep(0);
                   MessageBox (hwnd,
					            "Button: STD_REDOW",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_EDIT_DELETE:
                   MessageBeep(0);
                   MessageBox (hwnd,
					           "Button: STD_DELETE",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

				   
   				case ID_PRINT_PREVIEW:
                   MessageBeep(0);
                   MessageBox (hwnd,
					            "Button: STD_PRINTPRE",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_PRINT_PRINT:
                   MessageBeep(0);
                   MessageBox (hwnd,
					           "Button: STD_PRINT",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_PROPS:
                   MessageBeep(0);
                   MessageBox (hwnd,
					            "Button: STD_PROPERTIES",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

   				case ID_HELP:
                   MessageBeep(0);
                   MessageBox (hwnd,
					           "Button: STD_HELP",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;
				}

     		 break;

         //Toolbar autosizing provided
          case WM_SIZE:
               // obtain window size in pixels
               cxClient = LOWORD (lParam) ;
               cyClient = HIWORD (lParam) ;
               SendMessage (tbHandle, TB_AUTOSIZE, 0, 0L);
			   return 0 ;

          case WM_PAINT:
               hdc = BeginPaint (hwnd, &ps) ;



               EndPaint (hwnd, &ps) ;
               return 0 ;

          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }
     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
     }

