/**********************************************************************
  PROJECT: Toolbar Demo No 2
  Source: tb1_demo.cpp
  Chapter reference: 5
***********************************************************************
  Description:
    Toolbar demonstrations

  Topics:
    1. Combobox in toolbar
	2. Tooltip support
	3. Custom toolbar buttons using a bitmap
	4. Status bar display with toolbar button action
***********************************************************************/

#include <windows.h>
#include "resource.h"
#include <commctrl.h>
#define   MAX_MENUS 3              // Number of popup menus

// Prototype for WndProc()
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

// Glogal variables
HINSTANCE   pInstance;                // Handle to instance


/**************************
         WinMain()
***************************/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

	 wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW  ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
     wndclass.lpszClassName = szAppName ;
	 wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

	 pInstance = hInstance;

	 InitCommonControls();

     RegisterClassEx (&wndclass) ;

     hwnd = CreateWindow (szAppName, "Toolbar Demo No 2",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
         }
     return msg.wParam ;
     }

/******************************
      Windows Procedure
******************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{

     HDC			hdc ;
     static HMENU   menu1;       // handle to a menu
     TEXTMETRIC		tm ;

     static HWND   tbHandle, cbHandle, sbHandle; // Handles for toolbar,
	                                             // combobox and statusbar 
     static HWND   hwndTT;             // Handle for tooltip control

	 // Status bar general data
     int x, y, cx, cy;           // Status bar controls
     static RECT         sbWin;  // Rectangle structure for status bar window

	 // Status bar message strings displayed by button action or
	 // by highlighting menu entries
	 static char szFileNew[]   = "Creates a new disk file";
	 static char szFileOpen[]  = "Opens an existing disk file";
	 static char szFileSave[]  = "Saves the current file to disk";
	 static char szFileExit[]  = "Terminate program and return to Windows";
	 static char szNoData[]    = "No info for this entry";

     // Tooltip variables
     LPNMHDR          pnmh;      // Pointer to NMHDR
	 LPSTR            pReply;    // Pointer to string
	 LPTOOLTIPTEXT    lpttt;     // Pointer to TOOLTIPTEXT structure
	 BOOL             bSuccess;  // Boolean for call return 
	 static TOOLINFO  ti;        // TOOLINFO structure variable

	 // Data for statusbar update with menu selection 
	 UINT		fuFlags;                // Flags in WM_MENUSELECT
	 UINT       uItem;                  // Selecte menu item
     static HMENU		hMainMenu ;     // Another handle to a menu
     int		i ;						// counter

	 // Global data for status bar update operations
	 // Structure 
	 typedef struct tagPOPUPSTRING
		{
		HMENU hMenu ;
		UINT uiString ;
		} POPUPSTRING ;

	 static POPUPSTRING popstr[MAX_MENUS] ;

	 /************************************************
	   Array of attributes for buttons in the toolbar
      ************************************************/
	 TBBUTTON tbb[] = {
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0, // Spaces for combobox
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0, 
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, 0, 0, 0,
	 // Buttons deom bitmap
     0, MY_FILE_NEW,   TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
     1, MY_FILE_OPEN,  TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
     2, MY_FILE_SAVE,  TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
     0, 0,             TBSTATE_ENABLED, TBSTYLE_SEP,    0, 0, 0, 0,
	 3, ID_FILE_EXIT,  TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
     };



     static int  cxChar, cyChar, cxClient, cyClient;
     static int iMenu;
	 
     char *szStrings[] = { "Visual C++", "Borland C", "Pascal",
                           "Fortran 80", "Visual Basic" };

     switch (iMsg)
     {
	 case WM_CREATE: {

		      // Initialization operations
			  hdc = GetDC (hwnd) ;
			  GetTextMetrics (hdc, &tm) ;
			  cxChar = tm.tmAveCharWidth ;
			  cyChar = tm.tmHeight ;

			  /******************************
			       Create a toolbar
               ******************************/
			  tbHandle = CreateToolbarEx (hwnd,  // Handle to window
                         WS_CHILD | WS_VISIBLE | WS_BORDER |
						 TBSTYLE_TOOLTIPS | CCS_ADJUSTABLE, // Window styles
						 0,                          // Toolbar identifier
						 4,                          // Number of button images
						                             // in toolbar bitmap
						 pInstance,                      // Module instance
                         IDB_BITMAP1,                // Bitmap ID
			             tbb,                        // TBBUTON variable  
						 22,                         // Number of buttons
						                             // + separators
						 0, 0,                       // button dimensions (NR)
						 0, 0,                       // Bitmap dimensions (NR)
                         sizeof (TBBUTTON));
			  
			  // Create a combobox on the toolbar
			  cbHandle = CreateWindow ("COMBOBOX",
				         NULL,                       // No class name
						 WS_CHILD | WS_VISIBLE | WS_BORDER |
						 CBS_HASSTRINGS |CBS_DROPDOWNLIST,
						 0,                          // x origin
						 0,                          // y origin
						 130,                        // width   
						 144,                        // height
						 tbHandle,                   // Parent window 
						 (HMENU) IDR_MENU1,          // Menu resource ID
						 pInstance,                  // Application instance
						 0);
                //Add strings to combo box    
                for (i=0; i < 5; i++)
                    SendMessage(cbHandle,
					            CB_INSERTSTRING,
								(WPARAM)-1,
					            (LPARAM)szStrings[i]);

			  // Implement tooltip for combobox
			  hwndTT = (HWND) SendMessage((tbHandle), TB_GETTOOLTIPS, 0, 0L);

			  ZeroMemory (&ti, sizeof(TOOLINFO));
			  ti.cbSize = sizeof (TOOLINFO);
              ti.uFlags = TTF_IDISHWND | TTF_CENTERTIP | TTF_SUBCLASS;
			  ti.hwnd = tbHandle;
			  ti.uId = (UINT)(HWND) cbHandle;
			  ti.lpszText = LPSTR_TEXTCALLBACK;

			  bSuccess = SendMessage(hwndTT, TTM_ADDTOOL, 0,
				         (LPARAM)(LPTOOLINFO) &ti);
				  
			  // Create a status bar
			  sbHandle = CreateStatusWindow ( WS_CHILD | WS_VISIBLE |
						 WS_CLIPSIBLINGS | CCS_BOTTOM |
						 SBARS_SIZEGRIP,
						 "Ready",
						 hwnd,
						 2);

			  // Initialize status bar strings
			  menu1 = GetMenu(hwnd);         // Get handle to menu
			  popstr[0].hMenu = menu1;
			  popstr[0].uiString = IDS_FILEMENU;
			  
			  popstr[1].hMenu = GetSubMenu(menu1, 2);
			  popstr[1].uiString = IDS_EDITMENU;

			  popstr[2].hMenu = GetSubMenu(menu1, 3);
			  popstr[2].uiString = IDS_HELPMENU;



					ReleaseDC (hwnd, hdc) ;

                return 0 ;
			  } // end of WM_CREATE 

           case WM_COMMAND:
              menu1 = GetMenu(hwnd);
			    switch (LOWORD (wParam))
                {

                /**********************************
				   File menu entries processing
				 *********************************/
				case MY_FILE_NEW:
                   MessageBeep(0);
                   MessageBox (hwnd,
					           "New File Requested",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;
				   
                 case MY_FILE_OPEN:
                   MessageBeep(0);
                   MessageBox (hwnd,
					           "Open File Requested",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case MY_FILE_SAVE:
                   MessageBeep(0);
                   MessageBox (hwnd,
					           "Save File Requested",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case ID_FILE_EXIT:
					SendMessage(hwnd, WM_CLOSE, 0, 0L);
					return 0;

              
     			} // End of WM_COMMAND
				break;

         /**********************************
		        Tooltip processing
		  **********************************
		       pnmw is a variable of type NMHDR
			   lpttt is a variable of type TOOLTIPTEXT */
				
			  case WM_NOTIFY:
			  {
               pnmh = (LPNMHDR) lParam;

			  // Fetch tooltip text
			  if (pnmh->code == TTN_NEEDTEXT) {
				  lpttt = (LPTOOLTIPTEXT) lParam;
				  switch (lpttt->hdr.idFrom) {

				      case MY_FILE_NEW:
                      pReply = "New File";
					  // Update the status bar
					  SendMessage(sbHandle, SB_SETTEXT, 0,
						         (LPARAM)(LPSTR) szFileNew);
					  break;
                
                  	  case MY_FILE_OPEN:
                      pReply = "Open File";
					  // Update the status bar
					  SendMessage(sbHandle, SB_SETTEXT, 0,
						         (LPARAM)(LPSTR) szFileOpen);
					  break;

					  case MY_FILE_SAVE:
                      pReply = "Save File";
					  // Update the status bar
					  SendMessage(sbHandle, SB_SETTEXT, 0,
						         (LPARAM)(LPSTR) szFileSave);
					  break;
					  
					  case ID_FILE_EXIT:
                      pReply = "Exit program";
					  // Update the status bar
					  SendMessage(sbHandle, SB_SETTEXT, 0,
						         (LPARAM)(LPSTR) szFileExit);
					  break;

					  default:
					  // Combobox tooltip processing
                      if((lpttt->uFlags & TTF_IDISHWND) &&
						  (lpttt->hdr.idFrom == (UINT) cbHandle))
						   {
					       pReply = "Combo box";
					       }
					  else {
						   pReply = "Unknown";
						   }

                    }
                 lstrcpy (lpttt->szText, pReply); 
			     return 0;
			   }
			} // end of WM_NOTIFY
              
					

          case WM_SIZE:
               {
			  // obtain window size in pixels
               cxClient = LOWORD (lParam) ;
               cyClient = HIWORD (lParam) ;

			   // Resize status window
			   GetWindowRect(sbHandle, &sbWin);    // Get sb coordinates
			   cy = sbWin.bottom - sbWin.top;      // calculate height (cy)
			   x = 0;                              // x is at the start
			   y = cyClient - cy;                  // new y 
			   cx = cxClient;                      // x is same as client
			   // Move the status bar window
 			   MoveWindow(sbHandle,           // Handle to the sb
				   x, y,                      // New x/y position
				   cx, cy,                    // Window size
				   TRUE);                     // Repaint the window

			   // Resize toolbar
			   SendMessage (tbHandle, TB_AUTOSIZE, 0, 0L);

			   return 0 ;
               } // end of WM_SIZE


          /***************************************
		     status bar update on menu selection
           ***************************************
		   At this point:
			   HIWORD (wParam) hold the menu flags
			   (HMENU) lParam holds the handle to the clicked menu */
          case WM_MENUSELECT:
		  {
			  fuFlags = (UINT) HIWORD (wParam);   // Store flags
			  uItem = (UINT) LOWORD (wParam);     // Store menu item ID
			  hMainMenu = NULL ;                  // Dummy menu handle

 			/***************************
			   test for system menu
             *************************** 
			   fuFlags & MF_SYSMENU is true if a system menu is selected */
				if (fuFlags & MF_SYSMENU)
				{	
			    // Display helpful text in status bar using MenuHelp()
					MenuHelp (WM_MENUSELECT,      // Either WM_MENUSELECT
			 							          // or WM_COMMAND
					wParam,          
					lParam,
					menu1,                 // Handle to main menu
					pInstance,                 // Module instance
					sbHandle,              // Handle to statusbar window
					(UINT *) &popstr[0]) ; // Pointer to array
                    return 0;
			 	} // end of system menu handler

             /************************************ 
			   test for non-system && popup menu
			  ************************************
		       fuFlags & MF_POPUP is true if a popup menu is selected
			   System menus have been previously handled */
			     if (fuFlags & MF_POPUP)
			     {
					/* The following loop searches through all the popup menus
					   for a match between the handle of the clicked menu and
					   the one stored in the popstr[i].hMenu structure. See
					   structure intialization in WM_CREATE */
					for (i = 0 ; i < MAX_MENUS ; i++)
						{
						if ((HMENU) lParam == popstr[i].hMenu)
						  {
						  hMainMenu = (HMENU) lParam ;
			    		  // Display helpful text in status bar
						  MenuHelp (WM_MENUSELECT,
						  wParam,          
						  lParam,
						  menu1,                 // Handle to main menu ****
						  pInstance,                 // Module instance
						  sbHandle,              // Handle to statusbar window
						  (UINT *) &popstr[i]) ; // Pointer to array that has
												 // menu handles and resource
												 // string ID codes
                          break;

				           } // end of if
				        } // end of handle-matching loop
			      } // End of non-system && popup  menu handler

             /************************************ 
			   tests for highlighted menu entries
			  ************************************
			  In this case we must test for two conditions: that a menu item
			  is highlighted and that no popup menu is selected. This is
		      necessary because popup menus alse set the MF_HILITE flag, which
			  would destroy the corrsponding popup menu and display the default
			  one.	 */ 
			  if ((fuFlags & MF_HILITE) && (!(fuFlags & MF_POPUP)))
			     {
				  switch (uItem)
				  {
					  // Test for match and update status bar
				  	  case MY_FILE_NEW:
						SendMessage(sbHandle, SB_SETTEXT, 255,
						         (LPARAM)(LPSTR) szFileNew);
						break;
					                  
                  	  case MY_FILE_OPEN:
						SendMessage(sbHandle, SB_SETTEXT, 255,
						         (LPARAM)(LPSTR) szFileOpen);
						break;

					  case MY_FILE_SAVE:
						SendMessage(sbHandle, SB_SETTEXT, 255,
						         (LPARAM)(LPSTR) szFileSave);
						break;
					  
					  case ID_FILE_EXIT:
						SendMessage(sbHandle, SB_SETTEXT, 255,
						         (LPARAM)(LPSTR) szFileExit);
						break;

					  default:
						SendMessage(sbHandle, SB_SETTEXT, 255,
						         (LPARAM)(LPSTR) szNoData);
						break;

			       } // end of switch

			     return 0;  
			  } 
			}
		  break;

 
          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;

		}  // end of switch
     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
  }

