/**********************************************************************
  PROJECT: Tooltip Demo
  Source: tt_demo.cpp
  Chapter reference: 5
***********************************************************************
  Description:
    Tooltip demonstrations

  Topics:
    1. Implementing tooltip messages for toolbar buttons
***********************************************************************/


#include <windows.h>
#include "resource.h"
#include <commctrl.h>       // common controls header

// Prototype for WndProc()
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
HWND CreateAToolBar(HWND); 

// Glogal variables
HINSTANCE   pInstance;                // Handle to instance


/**************************
         WinMain()
***************************/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

	 wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW  ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
     wndclass.lpszClassName = szAppName ;
	 wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

	 pInstance = hInstance;

	 // Call common controls library
	 InitCommonControls();

     RegisterClassEx (&wndclass) ;

     hwnd = CreateWindow (szAppName, 
		                  "Tooltip Demo",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }

/******************************
      Windows Procedure
******************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
     {

     HDC             hdc ;

     TEXTMETRIC      tm ;
     static HWND     tbHandle;    // Handle to toolbar
     static HWND     hWndTT;      // Handle to tooltip
	 LPNMHDR         pnmh;


	 TOOLINFO        lpToolInfo;
	 LPTOOLTIPTEXT   lpToolTipText;
	 static char     szBuf [128];         // Buffer for tooltip text

	 /************************************************
	   Array of attributes for buttons in the toolbar
      ************************************************/
	 TBBUTTON tbb[] = {
     0, ID_FILE_NEW,   TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
     1, ID_FILE_OPEN,  TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
     2, ID_FILE_SAVE,  TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
	 };
 
     static int  cxChar, cyChar, cxClient, cyClient;

     switch (iMsg)
          {
		  case WM_CREATE:
			   // Initialization
               hdc = GetDC (hwnd) ;
               GetTextMetrics (hdc, &tm) ;
               cxChar = tm.tmAveCharWidth ;
               cyChar = tm.tmHeight ;

			  /******************************
			       Create a toolbar
               ******************************/
			  tbHandle = CreateToolbarEx (hwnd,  // Handle to window
                         WS_CHILD | WS_VISIBLE |
						 WS_CLIPSIBLINGS |
						 CCS_TOP | TBSTYLE_TOOLTIPS, // Window styles
						 0,                          // Toolbar identifier
						 3,                          // Number of button images
						                             // in toolbar bitmap
						 pInstance,                  // Module instance
                         IDB_BITMAP1,                // Bitmap ID
                         tbb,                        // TBBUTON structure  
						 3,                          // Number of buttons
						                             // + separators
						 0, 0,                       // button dimensions (NR)
						 0, 0,                       // Bitmap dimensions (NR)
                         sizeof (TBBUTTON));

			   // Get the handle to the tooltip window.
               hWndTT = (HWND)SendMessage(tbHandle, TB_GETTOOLTIPS, 0, 0);
               if (hWndTT) {
                 // Fill in the TOOLINFO structure.
                 lpToolInfo.cbSize = sizeof(lpToolInfo);
                 lpToolInfo.uFlags = TTF_IDISHWND | TTF_CENTERTIP;
                 lpToolInfo.hwnd = hwnd;
                 lpToolInfo.uId = (UINT)tbHandle;
                 lpToolInfo.hinst = pInstance;
                 lpToolInfo.lpszText = LPSTR_TEXTCALLBACK; 
			   }


               ReleaseDC (hwnd, hdc) ;
               return 0 ;

           case WM_COMMAND:
              //menu1 = GetMenu(hwnd);
			    switch (LOWORD (wParam))
                {
				case ID_FILE_NEW:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FILENEW",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;
				   
                 case ID_FILE_OPEN:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FILEOPEN",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;

                case ID_FILE_SAVE:
                   MessageBeep(0);
                   MessageBox (hwnd,
					   "Button: STD_FILESAVE",
							    "Menu Action",
								MB_ICONEXCLAMATION | MB_OK);
				   return 0;
				}

     		 break;

		case WM_NOTIFY:
			//switch (((LPNMHDR) lParam)->code) 
			pnmh = (LPNMHDR) lParam;
			switch (pnmh->code) {

			     case TTN_NEEDTEXT:    
                    // Display tool tip text.
			        lpToolTipText = (LPTOOLTIPTEXT)lParam;
    				LoadString (pInstance, 
	    				lpToolTipText->hdr.idFrom,    // Message origin
		    			szBuf,
			    		sizeof(szBuf));
				    	lpToolTipText->lpszText = szBuf;
			        break;

                default:
                    return TRUE;
                    break;
            }
			return 0;
			break;


          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }
     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
     }

