/**********************************************************************
  PROJECT: DC Info Demo
  Source: dci_demo.cpp
  Chapter reference: 6
***********************************************************************
  Description:
    Obtaining device context information

  Topics:
    1. Device capabilities using GetDeviceCaps()
	2. Obtaining the display driver version
	3. Obtaining line-drawing capabilites
	4. Obtaining curve drawing capabilites
***********************************************************************/

#include <windows.h>
#include "DC_Caps.h"       // Local array of structures
#include "resource.h"
#include <stdlib.h>        // for conversions
#include <string.h>        // for string copy

// Global variable for handle to program instance
HINSTANCE   pInstance;

// Predeclaration of the window procedure
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

//****************************
//         WinMain
//****************************
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     static char szAppName[] = "AppName" ; // Application name
     HWND        hwnd ;
     MSG         msg ;

     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
     wndclass.lpszClassName = szAppName ; 
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

     // Store handle to instance
	 pInstance = hInstance;
     
	 // Registering the structure wmdclass
     RegisterClassEx (&wndclass) ;

     // CreateWindow()
     hwnd = CreateWindow (szAppName,
           "DC Information Demo",         // window caption
            WS_OVERLAPPEDWINDOW,     // window style
            CW_USEDEFAULT,           // initial x position
            CW_USEDEFAULT,           // initial y position
            CW_USEDEFAULT,           // initial x size
            CW_USEDEFAULT,           // initial y size
            NULL,                    // parent window handle
            NULL,                    // window menu handle
            hInstance,               // program instance handle
            NULL) ;                  // creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     // Message loop
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
}
//****************************
//    Windows Procedure
//****************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam,
LPARAM lParam)
{
     static int  cxChar, cxCaps, cyChar, cxClient, cyClient ;

	 static int	 screenMode = 0; // Display mode control
	                       // 1 = general capabilities
						   // 2 = line capabilites
						   // 3 = curve capabilites
						   // 4 = version

     char        szBuffer[10] ;
     int         i;
	 int         linecaps, curvecaps;
	 char        szVersion[] = {"Driver version:               "};
	 char        szLineCaps[] = {"Line support       = YES \n"
		                         "Polyline drawing   = NO  \n"
								 "Marker drawing     = NO  \n"
								 "Polymarker drawing = NO  \n"
								 "Wide lines         = NO  \n"
								 "Styled lines       = NO  \n"
								 "Wide style lines   = NO  \n"
								 "Interior drawing   = NO  "};

	 char        szCurvCaps[] = {"Curve support      = YES \n"
		                         "Circle drawing     = NO  \n"
								 "Pie drawing        = NO  \n"
								 "Chord arcs drawing = NO  \n"
								 "Ellipses           = NO  \n"
								 "Wide borders       = NO  \n"
								 "Styled borders     = NO  \n"
								 "Wide and styled    = NO  \n"
								 "Interiors          = NO  \n"
								 "Rounded rectangles = NO  "};
	 
	 char        strNo[] = {"NO "};
	 char        strYes[]= {"YES"};

     static HDC         hdc ;
     PAINTSTRUCT ps ;
     TEXTMETRIC  tm ;
	 RECT		 textRect;                // RECT-type
	 HFONT       hFont;

     switch (iMsg)
         {
     // Windows message processing
	      case WM_CREATE:
			  hdc = GetDC(hwnd);    // Obtain handle to private DC
			   // Create a logical font
			   hFont = CreateFont (
					   12 * GetDeviceCaps (hdc, LOGPIXELSY) / 72, // height
					   0,                  // width
					   0,                  // escapement angle
					   0,                  // orientation angle
					   FW_NORMAL,          // weight
					   0,                  // not italics
					   0,                  // not underlined
					   0,                  // not strikedout
					   DEFAULT_CHARSET,    // character set
					   OUT_DEFAULT_PRECIS, // precision
					   CLIP_DEFAULT_PRECIS,// clipping precision
					   PROOF_QUALITY,      // quality
					   DEFAULT_PITCH | FF_DONTCARE, // pitch and family
					   "Courier");         // typeface name

			   SelectObject( hdc, hFont);  // Select font into DC

               // Text metrics are obtained after font is selected 
               GetTextMetrics (hdc, &tm) ;
               cxChar = tm.tmAveCharWidth ;
               cxCaps = (tm.tmPitchAndFamily & 1 ? 3 : 2) * cxChar / 2 ;
               cyChar = tm.tmHeight + tm.tmExternalLeading ;

              return 0;

          case WM_PAINT :
               BeginPaint (hwnd, &ps);
			   switch(screenMode)
			   {
			      case 1:
                  // Obtain and display DC capabilities
				  for (i = 0 ; i < LINES ; i++)
                    {
			        TextOut (hdc, cxChar, cyChar * (1 + i),
                             DCcaps[i].szLabel,
                             strlen (DCcaps[i].szLabel)) ;

                    TextOut (hdc, cxChar + 16 * cxCaps, cyChar * (1 + i),
                             DCcaps[i].szDesc,
                             strlen (DCcaps[i].szDesc)) ;

                    SetTextAlign (hdc, TA_RIGHT | TA_TOP) ;

                    TextOut (hdc, cxChar + 16 * cxCaps + 40 * cxChar,
			                 cyChar * (1 + i), szBuffer,
                             wsprintf (szBuffer, "%5d",
                                  GetDeviceCaps (hdc, DCcaps[i].iIndex))) ;

                    SetTextAlign (hdc, TA_LEFT | TA_TOP) ;
                    }
				  break;

			      case 2:
				  // Get capabilites
				  curvecaps = GetDeviceCaps(hdc, CURVECAPS);

				  if (curvecaps & CC_NONE)
					  strncpy(szCurvCaps + 21, strNo, 3);

				  if (curvecaps & CC_CIRCLES)
					  strncpy(szCurvCaps + (26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_PIE)
					  strncpy(szCurvCaps + (2 * 26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_CHORD)
					  strncpy(szCurvCaps + (3 * 26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_ELLIPSES)
					  strncpy(szCurvCaps + (4 * 26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_WIDE)
					  strncpy(szCurvCaps + (5 * 26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_STYLED)
					  strncpy(szCurvCaps + (6 * 26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_WIDESTYLED)
					  strncpy(szCurvCaps + (7 * 26 + 21), strYes, 3);

				  if (curvecaps & CC_INTERIORS)
					  strncpy(szCurvCaps + (8 * 26 + 21), strYes, 3);
				  
				  if (curvecaps & CC_ROUNDRECT)
					  strncpy(szCurvCaps + (9 * 26 + 21), strYes, 3);

				  
	              // Initialize rectangle structure
			      SetRect (&textRect,                // address of structure
			           2 * cxChar,               // x for start
					   cyChar,                   // y for start
					   cxClient,                 // x for end
					   cyClient);                // y for end

			      DrawText( hdc, szCurvCaps, -1, &textRect,
			              DT_LEFT | DT_WORDBREAK);
			      break;

			      case 3:
                  // Get capabilites
				  linecaps = GetDeviceCaps(hdc, LINECAPS);

				  if (linecaps & LC_NONE)
					  strncpy(szLineCaps + 21, strNo, 3);

				  if (linecaps & LC_POLYLINE)
					  strncpy(szLineCaps + (26 + 21), strYes, 3);
				  
				  if (linecaps & LC_MARKER)
					  strncpy(szLineCaps + (2 * 26 + 21), strYes, 3);
				  
				  if (linecaps & LC_POLYMARKER)
					  strncpy(szLineCaps + (3 * 26 + 21), strYes, 3);
				  
				  if (linecaps & LC_WIDE)
					  strncpy(szLineCaps + (4 * 26 + 21), strYes, 3);
				  
				  if (linecaps & LC_STYLED)
					  strncpy(szLineCaps + (5 * 26 + 21), strYes, 3);
				  
				  if (linecaps & LC_WIDESTYLED)
					  strncpy(szLineCaps + (6 * 26 + 21), strYes, 3);
				  
				  if (linecaps & LC_INTERIORS)
					  strncpy(szLineCaps + (7 * 26 + 21), strYes, 3);
				  
	              // Initialize rectangle structure
			      SetRect (&textRect,                // address of structure
			           2 * cxChar,               // x for start
					   cyChar,                   // y for start
					   cxClient,                 // x for end
					   cyClient);                // y for end

			      DrawText( hdc, szLineCaps, -1, &textRect,
			              DT_LEFT | DT_WORDBREAK);
			      break;

 		          case 4:
                  // Get driver version
				  _itoa(GetDeviceCaps(hdc, DRIVERVERSION),
					    szVersion + 16, 10);

	              // Initialize rectangle structure
			      SetRect (&textRect,                // address of structure
			           2 * cxChar,               // x for start
					   cyChar,                   // y for start
					   cxClient,                 // x for end
					   cyClient);                // y for end

			      DrawText( hdc, szVersion, -1, &textRect,
			              DT_LEFT | DT_WORDBREAK);

			      break;
			  }

			  EndPaint(hwnd, &ps);
			  break;

          case WM_SIZE:
			  // Determine and store size of client area
			  cxClient = LOWORD(lParam);
			  cyClient = HIWORD(lParam);
			  return 0;

          //********************************
          //     menu command processing
          //********************************
		  case WM_COMMAND:
			  switch (LOWORD (wParam))
			  {
			  case ID_DCINFO_CAPABILITES:
				  // Set display mode for this screen
				  screenMode = 1;
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

			  case ID_DCINFO_CURVECAPS:
				  // Set display mode for this screen
				  screenMode = 2;
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

			  case ID_DCINFO_LINECAPS:
				  // Set display mode for this screen
				  screenMode = 3;
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

			  case ID_DCINFO_VERSION:
				  // Set display mode for this screen
				  screenMode = 4;
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

			  case ID_DCINFO_EXIT:
				  SendMessage(hwnd, WM_DESTROY, 0, 0);
				  return 0;
		  }
		  return 0;

          // End of program execution
          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }
     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}
